<?php

namespace Drupal\slick\Entity;

use Drupal\blazy\Utility\NestedArray;

/**
 * Defines the Slick configuration entity.
 */
abstract class SlickBase implements SlickBaseInterface {

  use SlickPropertiesTrait;

  /**
   * Defines slick table name.
   */
  const TABLE = 'slick_optionset';

  /**
   * The legacy CTools ID for the configurable optionset.
   *
   * @var string
   */
  public $name;

  /**
   * The human-readable name for the optionset.
   *
   * @var string
   */
  public $label;

  /**
   * The plugin instance options.
   *
   * @var array
   */
  public $options = [];

  /**
   * The plugin default settings.
   *
   * @var array
   */
  protected static $defaultSettings;

  /**
   * Overrides Drupal\Core\Entity\Entity::id().
   */
  public function id() {
    return $this->name;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->label;
  }

  /**
   * Returns available slick default options under group 'settings'.
   *
   * @return array
   *   The default settings under options.
   */
  public static function defaultSettings() {
    return [];
  }

  /**
   * Returns default database field property values.
   *
   * @return mixed[]
   *   An array of property values, keyed by property name.
   */
  public static function defaultProperties() {
    return [
      'name'    => 'default',
      'label'   => 'Default',
      'options' => [],
    ];
  }

  /**
   * Returns the given optionset object identified by $id.
   *
   * @param string $id
   *   The optionset ID with property name, or default.
   *
   * @return object
   *   Returns the optionset, or else default, if no optionset found.
   */
  public static function load($id = 'default') {
    ctools_include('export');
    $optionset = ctools_export_crud_load(static::TABLE, $id);

    // Ensures deleted optionset while being used doesn't screw up.
    if (!isset($optionset->name)) {
      $optionset = ctools_export_crud_load(static::TABLE, 'default');
    }

    return $optionset;
  }

  /**
   * Load the optionset with a fallback.
   */
  public static function loadWithFallback($id) {
    $optionset = self::load($id);

    // Ensures deleted optionset while being used doesn't screw up.
    if (empty($optionset)) {
      $optionset = self::load('default');
    }
    return $optionset;
  }

  /**
   * Fetches all optionsets from the storage.
   *
   * @param bool $reset
   *   If TRUE, the static cache of all objects will be flushed prior to
   *   loading all. This can be important on listing pages where items
   *   might have changed on the page load.
   *
   * @return array
   *   The associative array of all optionsets.
   */
  public static function loadMultiple($reset = FALSE) {
    ctools_include('export');
    return ctools_export_crud_load_all(static::TABLE, $reset);
  }

  /**
   * Checks whether an optionset with the given name already exists.
   *
   * @param string $name
   *   The Optionset machine name.
   *
   * @return bool
   *   Returns TRUE if exists, FALSE otherwise.
   */
  public static function exists($name) {
    ctools_include('export');
    $optionset = ctools_export_crud_load(static::TABLE, $name);
    return isset($optionset->name);
  }

  /**
   * Returns a new optionset object without saving it to the database.
   *
   * @param array $values
   *   The values to build the optionset if provided.
   *
   * @return object
   *   Returns the programmatically created optionset object.
   */
  public static function create(array $values = []) {
    ctools_include('export');

    $optionset = ctools_export_crud_new(static::TABLE);

    $optionset->options = $optionset->options['settings'] = [];
    foreach (self::defaultProperties() as $key => $ignore) {
      if (isset($values[$key])) {
        $optionset->{$key} = $values[$key];
      }
    }

    if (empty($values['label']) && isset($values['name'])) {
      $optionset->label = $values['name'];
    }

    $defaults['settings'] = self::defaultSettings();
    $optionset->options = $optionset->options + $defaults;
    return $optionset;
  }

  /**
   * Returns the typecast values.
   *
   * @param array $settings
   *   An array of Optionset settings.
   */
  public static function typecast(array &$settings = []) {
    // Do nothing.
  }

  /**
   * {@inheritdoc}
   */
  public function delete() {
    ctools_include('export');
    ctools_export_crud_delete(static::TABLE, $this->name);
  }

  /**
   * {@inheritdoc}
   */
  public function save() {
    $data = $this->toArray();
    $update = self::exists($data['name']) ? ['name'] : [];

    $defaults['settings'] = self::defaultSettings();
    $data['options'] = $data['options'] + $defaults;

    return drupal_write_record(static::TABLE, $data, $update);
  }

  /**
   * {@inheritdoc}
   */
  public function getOptions($group = NULL, $property = NULL) {
    if ($group) {
      if (is_array($group)) {
        return NestedArray::getValue($this->options, (array) $group);
      }
      elseif (isset($property) && isset($this->options[$group])) {
        return $this->options[$group][$property] ?? NULL;
      }
      return $this->options[$group] ?? NULL;
    }

    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings($ansich = FALSE) {
    if ($ansich && isset($this->options['settings'])) {
      return $this->options['settings'];
    }

    // With the Optimized options, all defaults are cleaned out, merge em.
    return isset($this->options['settings']) ? NestedArray::mergeDeep(self::defaultSettings(), $this->options['settings']) : self::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function setSettings(array $settings = []) {
    $this->options['settings'] = $settings;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSetting($name) {
    return $this->getSettings()[$name] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setSetting($name, $value) {
    $this->options['settings'][$name] = $value;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function toArray() {
    $values = [];
    foreach (self::defaultProperties() as $key => $ignore) {
      $values[$key] = $this->{$key};
    }
    return $values;
  }

}
